import PocketBase from 'pocketbase';
import type { User, Product, CartItem, Favorite, Order, AuthResponse, ApiError } from './types';

// Initialize PocketBase client
const pb = new PocketBase('https://pb.vinylnostalgia.com/'); // Replace with your PocketBase URL

// Helper function to handle errors
// eslint-disable-next-line @typescript-eslint/no-explicit-any
const handleError = (error: any): ApiError => {
	console.error('API Error:', error);
	throw { message: error.message, code: error.status, details: error.data };
};

// User Authentication
export const authService = {
	// Register a new user
	register: async (email: string, password: string, name: string): Promise<User> => {
		try {
			const user = await pb.collection('users').create({
				email,
				password,
				passwordConfirm: password,
				name
			});
			return user as unknown as User; // Explicit type casting
		} catch (error) {
			throw handleError(error);
		}
	},

	// Login a user
	login: async (email: string, password: string): Promise<AuthResponse> => {
		try {
			const authData = await pb.collection('users').authWithPassword(email, password);
			return {
				token: authData.token,
				user: authData.record as unknown as User // Explicit type casting
			};
		} catch (error) {
			throw handleError(error);
		}
	},

	// Logout the current user
	logout: (): void => {
		pb.authStore.clear();
	},

	// Get the current user
	getCurrentUser: (): User | null => {
		return pb.authStore.model as unknown as User | null; // Explicit type casting
	},

	// Check if the user is authenticated
	isAuthenticated: (): boolean => {
		return pb.authStore.isValid;
	}
};

// Product Service
export const productService = {
	// Fetch all products
	getProducts: async (): Promise<Product[]> => {
		try {
			const products = await pb.collection('products').getFullList();
			return products as unknown as Product[]; // Explicit type casting
		} catch (error) {
			throw handleError(error);
		}
	},

	// Fetch a single product by ID
	getProductById: async (id: string): Promise<Product> => {
		try {
			const product = await pb.collection('products').getOne(id);
			return product as unknown as Product; // Explicit type casting
		} catch (error) {
			throw handleError(error);
		}
	}
};

// Cart Service
export const cartService = {
	// Add a product to the cart
	addToCart: async (userId: string, productId: string, quantity: number): Promise<CartItem> => {
		try {
			const cartItem = await pb.collection('cart').create({
				user: userId,
				product: productId,
				quantity
			});
			return cartItem as unknown as CartItem; // Explicit type casting
		} catch (error) {
			throw handleError(error);
		}
	},

	// Remove a product from the cart
	removeFromCart: async (cartItemId: string): Promise<void> => {
		try {
			await pb.collection('cart').delete(cartItemId);
		} catch (error) {
			throw handleError(error);
		}
	},

	// Update the quantity of a cart item
	updateCartItemQuantity: async (cartItemId: string, quantity: number): Promise<CartItem> => {
		try {
			const updatedItem = await pb.collection('cart').update(cartItemId, { quantity });
			return updatedItem as unknown as CartItem; // Explicit type casting
		} catch (error) {
			throw handleError(error);
		}
	},

	// Fetch the user's cart
	getCart: async (userId: string): Promise<CartItem[]> => {
		try {
			const cartItems = await pb.collection('cart').getFullList({
				filter: `user = "${userId}"`,
				expand: 'product'
			});
			return cartItems as unknown as CartItem[]; // Explicit type casting
		} catch (error) {
			throw handleError(error);
		}
	},

	// Clear the user's cart
	clearCart: async (userId: string): Promise<void> => {
		try {
			const cartItems = await pb.collection('cart').getFullList({
				filter: `user = "${userId}"`
			});
			for (const item of cartItems) {
				await pb.collection('cart').delete(item.id);
			}
		} catch (error) {
			throw handleError(error);
		}
	}
};

// Favorites Service
export const favoritesService = {
	// Add a product to favorites
	addToFavorites: async (userId: string, productId: string): Promise<Favorite> => {
		try {
			const favorite = await pb.collection('favorites').create({
				user: userId,
				product: productId
			});
			return favorite as unknown as Favorite; // Explicit type casting
		} catch (error) {
			throw handleError(error);
		}
	},

	// Remove a product from favorites
	removeFromFavorites: async (favoriteId: string): Promise<void> => {
		try {
			await pb.collection('favorites').delete(favoriteId);
		} catch (error) {
			throw handleError(error);
		}
	},

	// Fetch the user's favorites
	getFavorites: async (userId: string): Promise<Favorite[]> => {
		try {
			const favorites = await pb.collection('favorites').getFullList({
				filter: `user = "${userId}"`,
				expand: 'product'
			});
			return favorites as unknown as Favorite[]; // Explicit type casting
		} catch (error) {
			throw handleError(error);
		}
	}
};

// Order Service
export const orderService = {
	// Create a new order
	createOrder: async (
		userId: string,
		items: Array<{ product: string; quantity: number }>,
		total: number
	): Promise<Order> => {
		try {
			const order = await pb.collection('orders').create({
				user: userId,
				items,
				total,
				status: 'pending'
			});
			return order as unknown as Order; // Explicit type casting
		} catch (error) {
			throw handleError(error);
		}
	},

	// Fetch the user's orders
	getOrders: async (userId: string): Promise<Order[]> => {
		try {
			const orders = await pb.collection('orders').getFullList({
				filter: `user = "${userId}"`,
				expand: 'items.product'
			});
			return orders as unknown as Order[]; // Explicit type casting
		} catch (error) {
			throw handleError(error);
		}
	}
};

// Export all services
export default {
	authService,
	productService,
	cartService,
	favoritesService,
	orderService
};
