import { writable } from 'svelte/store';

// Define the type for a product
export interface Product {
	id: number;
	title: string;
	description: string;
	imageUrl: string;
	price: number; // Added price for cart calculations
	quantity?: number; // Optional quantity for cart items
}

// Define the type for the cart and favorites
export interface AppState {
	cart: Product[];
	favorites: Product[];
}

// Initialize the store
const initialState: AppState = {
	cart: [],
	favorites: []
};

export const appStore = writable<AppState>(initialState);
export const favorites = writable<Product[]>([]);

// Add a product to the cart
export const addToCart = (product: Product) => {
	appStore.update((state) => {
		const existingItem = state.cart.find((item) => item.id === product.id);
		if (existingItem) {
			// If the product already exists, increase the quantity
			return {
				...state,
				cart: state.cart.map((item) =>
					item.id === product.id ? { ...item, quantity: (item.quantity || 1) + 1 } : item
				)
			};
		} else {
			// If the product doesn't exist, add it to the cart with quantity 1
			return { ...state, cart: [...state.cart, { ...product, quantity: 1 }] };
		}
	});
};

// Remove a product from the cart
export const removeFromCart = (productId: number) => {
	appStore.update((state) => ({
		...state,
		cart: state.cart.filter((item) => item.id !== productId)
	}));
};

// Update the quantity of a product in the cart
export const updateCartQuantity = (productId: number, quantity: number) => {
	appStore.update((state) => ({
		...state,
		cart: state.cart.map((item) => (item.id === productId ? { ...item, quantity } : item))
	}));
};

// Add a product to favorites
export const addToFavorites = (product: Product) => {
	favorites.update((items) => {
		if (!items.some((item) => item.id === product.id)) {
			return [...items, product];
		}
		return items;
	});
};

// Remove a product from favorites
export const removeFromFavorites = (productId: number) => {
	favorites.update((items) => items.filter((item) => item.id !== productId));
};

// Clear the cart
export const clearCart = () => {
	appStore.update((state) => ({ ...state, cart: [] }));
};

// Clear favorites
export const clearFavorites = () => {
	favorites.update((state) => ({ ...state, favorites: [] }));
};
